function F = F_choreographies(x,tu,par)

% Inputs: x = [lambda;alpha;u;v;w]; 
%               lambda = (lambda_1,lambda_2,lambda_3) in C^3 the unfolding
%                        parameters for the three conserved quantities
%               alpha = (alpha_1,...,lambda_{n-1}) in C^{n-1} the unfolding
%                        parameters for the n-1 extra initial conditions
%               u = (u_1,u_2,u_3) in (ell_nu^1)^3 the Fourier coefficients
%                   of the original phase space variables u(t) = (x(t),y(t),z(t))
%               v = (v_1,v_2,v_3) in (ell_nu^1)^3 the Fourier coefficients
%                   of the derivative v(t) = u'(t) = (x'(t),y'(t),z'(t))
%               w = (w_1,...,w_{n-1}) in (ell_nu^1)^{n-1} the Fourier coefficients
%                   of the nonlinearity functions w_j(t)
%         tu = (tu_1,tu_2,tu_3) the Fourier coefficients of a periodic
%              solution previously obtained 
%         par = [omega;n;k];
%
% Outputs : F = [eta;gamma;f;g;h];
%
%           eta = (eta1,eta2,eta3) in C^3 the three phase conditions
%           gamma = the extra initial conditions
%           f = f(u,v) = Du - v
%           g = g(lambda,u,v,w,omega) = omega^2*Dv +2*omega*sqrt(s1)*bJ*v - ...
%           h = h(alpha,u,v,w)

omega = par(1); % Frequency of the periodic orbit
n = par(2); % Number of bodies in the choreography
k = par(3); % Defines the symmetry of the choreography
zeta = 2*pi/n;
s1 = (1/4)*sum(1./sin((1:n-1)*zeta/2));

% lambda in C^3: unfolding parameters for the three conserved quantities
lambda = x(1:3); 

%alpha in C^{n-1}: unfolding parameters for the n-1 extra init. conditions
alpha = x(4:n+2);

m = (length(x)+3)/(2*(n+5)); % The Fourier projection per component

%u : Fourier coefficients of u(t) = (x(t),y(t),z(t))
u = reshape(x(n+3:6*m+n-1),2*m-1,3); %u_1 = u(:,1); u_2 = u(:,2); u_3 = u(:,3);
u1 = u(:,1); u2 = u(:,2); u3 = u(:,3);

%v : Fourier coefficients of v(t) = u'(t) = (x'(t),y'(t),z'(t))
v = reshape(x(6*m+n:12*m+n-4),2*m-1,3); 
v1 = v(:,1); v2 = v(:,2); v3 = v(:,3);

%w : Fourier coefficients of w(t) = (w_1(t),...,w_{n-1}(t))
w = reshape(x(12*m+n-3:2*m*(n+5)-3),2*m-1,n-1); %w_1 = w(:,1); ... w_{n-1} = w(:,n-1);

i1 = (1:2*m-1)'; i2 = i1+2*m-1; i3 = i2+2*m-1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% eta - Conserved quantities %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tu1 = tu(i1); tu2 = tu(i2); tu3 = tu(i3);

ell = (-m+1:m-1)';

I1 = -transpose(u1)*flip(tu2)+transpose(u2)*flip(tu1);
tu1p = 1i*ell.*tu1; tu2p = 1i*ell.*tu2; tu3p = 1i*ell.*tu3;
I2 = transpose(u1)*flip(tu1p)+transpose(u2)*flip(tu2p)+transpose(u3)*flip(tu3p);
I3 = u3(m);

eta = [I1;I2;I3];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% gamma - extra initial conition %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gamma = zeros(n-1,1);

for j=1:n-1

    Mju = M_j([u1;u2;u3],j,k,zeta);
    
    Mju1 = Mju(i1); Mju2 = Mju(i2); Mju3 = Mju(i3);
    
    wj = w(:,j);
    
    gamma(j) = sum(wj)^2*(sum(Mju1)^2+sum(Mju2)^2+sum(Mju3)^2)-1;
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% f = f(u,v) = Du - v %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%

f = [1i*ell.*u1-v1;1i*ell.*u2-v2;1i*ell.*u3-v3];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% g = g(u,v,w,omega) = omega^2*Dv + ... %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

one = zeros(2*m-1,1); one(m) = 1;

g1_linear = omega^2*1i*ell.*v1 - 2*omega*sqrt(s1)*v2 - s1*u1 - lambda(1)*u2 + lambda(2)*v1;
g2_linear = omega^2*1i*ell.*v2 + 2*omega*sqrt(s1)*v1 - s1*u2 + lambda(1)*u1 + lambda(2)*v2;
g3_linear = omega^2*1i*ell.*v3                                              + lambda(2)*v3 + lambda(3)*one ;

g1_nonlinear = zeros(2*m-1,1); g2_nonlinear = zeros(2*m-1,1); g3_nonlinear = zeros(2*m-1,1);

for j = 1:n-1
    
    wj = w(:,j);
    
    Mju = M_j([u1;u2;u3],j,k,zeta);
    
    Mju1 = Mju(i1); Mju2 = Mju(i2); Mju3 = Mju(i3);
    
    Mju1wj3 = quartic(Mju1,wj,wj,wj);
    Mju2wj3 = quartic(Mju2,wj,wj,wj);
    Mju3wj3 = quartic(Mju3,wj,wj,wj);
        
    g1_nonlinear = g1_nonlinear + Mju1wj3;
    g2_nonlinear = g2_nonlinear + Mju2wj3;
    g3_nonlinear = g3_nonlinear + Mju3wj3;
    
end

g = [ g1_linear + g1_nonlinear ; g2_linear + g2_nonlinear ; g3_linear + g3_nonlinear ];

%%%%%%%%%%%%%%%%%%%%
%%% h = h(u,v,w) %%%
%%%%%%%%%%%%%%%%%%%%

h = zeros((2*m-1)*(n-1),1);

index = (1:2*m-1)';

for j = 1 : n-1
    
    wj = w(:,j);
    
    Mju = M_j([u1;u2;u3],j,k,zeta);
    Mjv = M_j([v1;v2;v3],j,k,zeta);
    
    wj3_Mju1_Mjv1 = quintic(wj,wj,wj,Mju(i1),Mjv(i1));
    wj3_Mju2_Mjv2 = quintic(wj,wj,wj,Mju(i2),Mjv(i2));
    wj3_Mju3_Mjv3 = quintic(wj,wj,wj,Mju(i3),Mjv(i3));
    
    wj3 = cubic(wj,wj,wj);
     
    h(index) = 1i*ell.*wj + wj3_Mju1_Mjv1 + wj3_Mju2_Mjv2 + wj3_Mju3_Mjv3 + alpha(j)*wj3;
    
    index = index + 2*m-1;
   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% F = [eta;gamma;f;g;h] %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

F = [eta;gamma;f;g;h];

end

