function iDF = int_DF_choreographies(x,tu,par)

% Inputs: x = [lambda;alpha;u;v;w];
%               lambda = (lambda_1,lambda_2,lambda_3) in C^3 the unfolding
%                        parameters for the three conserved quantities
%               alpha = (alpha_1,...,lambda_{n-1}) in C^{n-1} the unfolding
%                        parameters for the n-1 extra initial conditions
%               u = (u_1,u_2,u_3) in (ell_nu^1)^3 the Fourier coefficients
%                   of the original phase space variables u(t) = (x(t),y(t),z(t))
%               v = (v_1,v_2,v_3) in (ell_nu^1)^3 the Fourier coefficients
%                   of the derivative v(t) = u'(t) = (x'(t),y'(t),z'(t))
%               w = (w_1,...,w_{n-1}) in (ell_nu^1)^{n-1} the Fourier coefficients
%                   of the nonlinearity functions w_j(t)
%         tu = (tu_1,tu_2,tu_3) the Fourier coefficients of a periodic
%              solution previsouily obtained
%         par = [omega;n;k];

omega = par(1); % Frequency of the periodic orbit
n = mid(par(2)); % Number of bodies in the choreography
k = mid(par(3)); % Defines the symmetry of the choreography
zeta = 2*intval('pi')/n;
s1 = sum(1./sin((1:n-1)*zeta/2))/4;

x = intval(x);
tu = intval(tu);
omega = intval(omega);

% lambda in C^3: unfolding parameters for the three conserved quantities
lambda = x(1:3);

%alpha in C^{n-1}: unfolding parameters for the n-1 extra init. conditions
alpha = x(4:n+2);

m = (length(x)+3)/(2*(n+5)); % The Fourier projection per component

%u : Fourier coefficients of u(t) = (x(t),y(t),z(t))
u = reshape(x(n+3:6*m+n-1),2*m-1,3); %u_1 = u(:,1); u_2 = u(:,2); u_3 = u(:,3);
u1 = u(:,1); u2 = u(:,2); u3 = u(:,3);

%v : Fourier coefficients of v(t) = u'(t) = (x'(t),y'(t),z'(t))
v = reshape(x(6*m+n:12*m+n-4),2*m-1,3);
v1 = v(:,1); v2 = v(:,2); v3 = v(:,3);

%w : Fourier coefficients of w(t) = (w_1(t),...,w_{n-1}(t))
w = reshape(x(12*m+n-3:2*m*(n+5)-3),2*m-1,n-1); %w_1 = w(:,1); ... w_{n-1} = w(:,n-1);

i1 = (1:2*m-1)'; i2 = i1+2*m-1; i3 = i2+2*m-1;

%%%% Define the indices corresponding to the variables

i_lambda = (1:3);
i_alpha = i_lambda(end) + (1:n-1);
i_u1 = i_alpha(end) + (1:2*m-1);
i_u2 = i_u1(end) + (1:2*m-1);
i_u3 = i_u2(end) + (1:2*m-1);
i_v1 = i_u3(end) + (1:2*m-1);
i_v2 = i_v1(end) + (1:2*m-1);
i_v3 = i_v2(end) + (1:2*m-1);

i_wj = zeros(n-1,2*m-1);
i_wj(1,:) = i_v3(end) + (1:2*m-1);

for j=1:n-1
    i_wj(j+1,:) = i_wj(j,end) + (1:2*m-1);
end

%%%% Preallocate memory

iDF = intval(zeros(2*m*(n+5)-3));

%%%%%%%%%%%%%%%%%
%%% d_eta/d_u %%%
%%%%%%%%%%%%%%%%%

tu1 = tu(i1); tu2 = tu(i2); tu3 = tu(i3);

eta1_u = intval(zeros(1,4*m-2));
eta1_u(i1) = -flip(tu2); eta1_u(i2) = flip(tu1);
iDF(1,[i_u1 i_u2]) = eta1_u;

ell = (-m+1:m-1)';
eta2_u = intval(zeros(1,6*m-3));
eta2_u(i1) = -1i*ell.*flip(tu1); eta2_u(i2) = -1i*ell.*flip(tu2); eta2_u(i3) = -1i*ell.*flip(tu3);
iDF(2,[i_u1 i_u2 i_u3]) = eta2_u;

eta3_u3 = zeros(1,2*m-1); eta3_u3(m)=1;
iDF(3,[i_u1 i_u2 i_u3]) = intval([zeros(1,2*m-1) zeros(1,2*m-1) eta3_u3]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% d_gamma/d_u & d_gamma/d_wj %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j=1:n-1
    
    gamma_j_u = intval(zeros(1,6*m-3));
    
    wj = w(:,j);
    
    vec = exp(1i*j*k*ell*zeta);
    tvec = transpose(vec);
    
    Mju1 = u1 - cos(j*zeta)*vec.*u1 + sin(j*zeta)*vec.*u2;
    Mju2 = u2 - sin(j*zeta)*vec.*u1 - cos(j*zeta)*vec.*u2;
    Mju3 = u3 - vec.*u3;
    
    sum1 = real(sum(Mju1)); % for real symmetric data, we know theoretically that imag part = 0
    sum2 = real(sum(Mju2)); % for real symmetric data, we know theoretically that imag part = 0
    sum3 = real(sum(Mju3)); % for real symmetric data, we know theoretically that imag part = 0
    
    gamma_j_u(i1) = 2*sum(wj)^2*(sum1*(1 - cos(j*zeta)*tvec) + sum2*(-sin(j*zeta)*tvec));
    gamma_j_u(i2) = 2*sum(wj)^2*(sum1*(sin(j*zeta)*tvec) + sum2*(1 - cos(j*zeta)*tvec));
    gamma_j_u(i3) = 2*sum(wj)^2*(sum3*(1 - tvec));
    
    iDF(3+j,[i_u1 i_u2 i_u3]) = gamma_j_u;
    
    gamma_j_wj = 2*sum(wj)*(sum1^2+sum2^2+sum3^2)*ones(1,2*m-1);
    
    iDF(3+j,i_wj(j,:)) = gamma_j_wj;
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%%% d_f/d_u & d_f/d_v %%%
%%%%%%%%%%%%%%%%%%%%%%%%%

iDF(i_u1,i_u1) = diag(1i*ell); % d_f1/d_u1
iDF(i_u2,i_u2) = diag(1i*ell); % d_f2/d_u2
iDF(i_u3,i_u3) = diag(1i*ell); % d_f3/d_u3

iDF(i_u1,i_v1) = -eye(2*m-1); % d_f1/d_v1
iDF(i_u2,i_v2) = -eye(2*m-1); % d_f2/d_v2
iDF(i_u3,i_v3) = -eye(2*m-1); % d_f3/d_v3

%%%%%%%%%%%%%%%%%%%%
%%% d_g/d_lambda %%%
%%%%%%%%%%%%%%%%%%%%

d_gd_lambda1 = [-u2;u1;zeros(2*m-1,1)];
d_gd_lambda2 = [v1;v2;v3];

one = zeros(2*m-1,1); one(m) = 1;
d_gd_lambda3 = [zeros(2*m-1,1);zeros(2*m-1,1);one];

iDF([i_v1 i_v2 i_v3],1) = d_gd_lambda1;
iDF([i_v1 i_v2 i_v3],2) = d_gd_lambda2;
iDF([i_v1 i_v2 i_v3],3) = d_gd_lambda3;

%%%%%%%%%%%%%%%
%%% d_g/d_u %%%
%%%%%%%%%%%%%%%

w3_ext = intval(zeros(4*m-3,n-1));

for j = 1:n-1
    wj = w(:,j);
    wj3 = rig_conv_2sided([wj wj wj]); % size 6*(m-1)+1
    w3_ext(:,j) = wj3(m:5*m-4);
end

dg1_du1 = intval(zeros(2*m-1));
dg1_du2 = intval(zeros(2*m-1));
dg2_du1 = intval(zeros(2*m-1));
dg2_du2 = intval(zeros(2*m-1));
dg3_du3 = intval(zeros(2*m-1));

ell=(-m+1:m-1);

for s=-m+1:m-1
    for j = 1:n-1
        
        dg1_du1(ell+m,s+m) = dg1_du1(ell+m,s+m) + (1-exp(1i*j*k*s*zeta)*cos(j*zeta))*w3_ext(ell-s+2*m-1,j) ;
        dg1_du2(ell+m,s+m) = dg1_du2(ell+m,s+m) + exp(1i*j*k*s*zeta)*sin(j*zeta)*w3_ext(ell-s+2*m-1,j);
        
        dg2_du1(ell+m,s+m) = dg2_du1(ell+m,s+m) - exp(1i*j*k*s*zeta)*sin(j*zeta)*w3_ext(ell-s+2*m-1,j);
        dg2_du2(ell+m,s+m) = dg2_du2(ell+m,s+m) + (1-exp(1i*j*k*s*zeta)*cos(j*zeta))*w3_ext(ell-s+2*m-1,j);
        
        dg3_du3(ell+m,s+m) = dg3_du3(ell+m,s+m) + (1-exp(1i*j*k*s*zeta))*w3_ext(ell-s+2*m-1,j);
        
    end
end

dg1_du1 = -s1*eye(2*m-1) + dg1_du1;
iDF(i_v1,i_u1) = dg1_du1;

dg1_du2 = -lambda(1)*eye(2*m-1) + dg1_du2;
iDF(i_v1,i_u2) = dg1_du2;

dg2_du1 = lambda(1)*eye(2*m-1) + dg2_du1;
iDF(i_v2,i_u1) = dg2_du1;

dg2_du2 = -s1*eye(2*m-1) + dg2_du2;
iDF(i_v2,i_u2) = dg2_du2;

iDF(i_v3,i_u3) = dg3_du3;

%%%%%%%%%%%%%%%
%%% d_g/d_v %%%
%%%%%%%%%%%%%%%

iDF(i_v1,i_v1) = diag(omega^2*1i*ell) + lambda(1)*eye(2*m-1); % d_g1/d_v1
iDF(i_v1,i_v2) = - 2*omega*sqrt(s1)*eye(2*m-1); % d_g1/d_v2

iDF(i_v2,i_v1) = 2*omega*sqrt(s1)*eye(2*m-1); % d_g2/d_v1
iDF(i_v2,i_v2) = diag(omega^2*1i*ell) + lambda(2)*eye(2*m-1); % d_g2/d_v2

iDF(i_v3,i_v3) = diag(omega^2*1i*ell) + lambda(2)*eye(2*m-1); % d_g3/d_v3

%%%%%%%%%%%%%%%
%%% d_g/d_w %%%
%%%%%%%%%%%%%%%

w2Mju1_ext = intval(zeros(4*m-3,n-1));
w2Mju2_ext = intval(zeros(4*m-3,n-1));
w2Mju3_ext = intval(zeros(4*m-3,n-1));

for j = 1:n-1
    
    Mju = M_j([u1;u2;u3],j,k,zeta);
    Mju1 = Mju(i1); Mju2 = Mju(i2); Mju3 = Mju(i3);
    wj = w(:,j);
    
    w2Mju1 = rig_conv_2sided([wj wj Mju1]);
    w2Mju2 = rig_conv_2sided([wj wj Mju2]);
    w2Mju3 = rig_conv_2sided([wj wj Mju3]);
    
    w2Mju1_ext(:,j) = w2Mju1(m:5*m-4);
    w2Mju2_ext(:,j) = w2Mju2(m:5*m-4);
    w2Mju3_ext(:,j) = w2Mju3(m:5*m-4);
    
end

for j = 1:n-1
    
    dg1_dwj = intval(zeros(2*m-1)); dg2_dwj = intval(zeros(2*m-1)); dg3_dwj = intval(zeros(2*m-1));
    
    for s=-m+1:m-1
        
        dg1_dwj(ell+m,s+m) = 3*w2Mju1_ext(ell-s+2*m-1,j);
        dg2_dwj(ell+m,s+m) = 3*w2Mju2_ext(ell-s+2*m-1,j);
        dg3_dwj(ell+m,s+m) = 3*w2Mju3_ext(ell-s+2*m-1,j);
        
    end
    
    iDF(i_v1,i_wj(j,:)) = dg1_dwj;
    iDF(i_v2,i_wj(j,:)) = dg2_dwj;
    iDF(i_v3,i_wj(j,:)) = dg3_dwj;
    
end

%%%%%%%%%%%%%%%%%%%
%%% d_h/d_alpha %%%
%%%%%%%%%%%%%%%%%%%

for j = 1:n-1
    
    wj = w(:,j);
    
    wj3_ext = rig_conv_2sided([wj wj wj]); % size 6*(m-1)+1
    wj3 = wj3_ext(2*m-1:4*m-3);
    
    iDF(i_wj(j,:),j+3) = wj3;
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% d_h/d_u, d_h/d_v & d_h/d_w %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j=1:n-1
    
    dhj_du1 = intval(zeros(2*m-1)); dhj_du2 = intval(zeros(2*m-1)); dhj_du3 = intval(zeros(2*m-1));
    dhj_dv1 = intval(zeros(2*m-1)); dhj_dv2 = intval(zeros(2*m-1)); dhj_dv3 = intval(zeros(2*m-1));
    dhj_dwj = intval(zeros(2*m-1));
    
    Mju = M_j([u1;u2;u3],j,k,zeta);
    Mjv = M_j([v1;v2;v3],j,k,zeta);
        
    wj = w(:,j);
    
    Mjv1_wj3_ext = rig_conv_2sided([Mjv(i1) wj wj wj]); % size 8*(m-1)+1 = 8*m-6-1 = 2*(4*m-3)-1
    Mjv1_wj3 = Mjv1_wj3_ext(2*m-1:6*m-5);
    
    Mjv2_wj3_ext = rig_conv_2sided([Mjv(i2) wj wj wj]);
    Mjv2_wj3 = Mjv2_wj3_ext(2*m-1:6*m-5);
    
    Mjv3_wj3_ext = rig_conv_2sided([Mjv(i3) wj wj wj]);
    Mjv3_wj3 = Mjv3_wj3_ext(2*m-1:6*m-5);
    
    Mju1_wj3_ext = rig_conv_2sided([Mju(i1) wj wj wj]);
    Mju1_wj3 = Mju1_wj3_ext(2*m-1:6*m-5);
    
    Mju2_wj3_ext = rig_conv_2sided([Mju(i2) wj wj wj]);
    Mju2_wj3 = Mju2_wj3_ext(2*m-1:6*m-5);
    
    Mju3_wj3_ext = rig_conv_2sided([Mju(i3) wj wj wj]);
    Mju3_wj3 = Mju3_wj3_ext(2*m-1:6*m-5);
    
    wj2_ext = rig_conv_2sided([wj wj]);
    
    wj2_Mju1_Mjv1_ext = rig_conv_2sided([wj wj Mju(i1) Mjv(i1)]);
    wj2_Mju1_Mjv1 = wj2_Mju1_Mjv1_ext(2*m-1:6*m-5);
    
    wj2_Mju2_Mjv2_ext = rig_conv_2sided([wj wj Mju(i2) Mjv(i2)]);
    wj2_Mju2_Mjv2 = wj2_Mju2_Mjv2_ext(2*m-1:6*m-5);
    
    wj2_Mju3_Mjv3_ext = rig_conv_2sided([wj wj Mju(i3) Mjv(i3)]);
    wj2_Mju3_Mjv3 = wj2_Mju3_Mjv3_ext(2*m-1:6*m-5);
    
    for s=-m+1:m-1
        
        dhj_du1(ell+m,s+m) = (1-exp(1i*j*k*s*zeta)*cos(j*zeta))*Mjv1_wj3(ell-s+2*m-1) - exp(1i*j*k*s*zeta)*sin(j*zeta)*Mjv2_wj3(ell-s+2*m-1);
        dhj_du2(ell+m,s+m) = exp(1i*j*k*s*zeta)*sin(j*zeta)*Mjv1_wj3(ell-s+2*m-1) + (1-exp(1i*j*k*s*zeta)*cos(j*zeta))*Mjv2_wj3(ell-s+2*m-1);
        dhj_du3(ell+m,s+m) = (1-exp(1i*j*k*s*zeta))*Mjv3_wj3(ell-s+2*m-1);
        
        dhj_dv1(ell+m,s+m) = (1-exp(1i*j*k*s*zeta)*cos(j*zeta))*Mju1_wj3(ell-s+2*m-1) - exp(1i*j*k*s*zeta)*sin(j*zeta)*Mju2_wj3(ell-s+2*m-1);
        dhj_dv2(ell+m,s+m) = exp(1i*j*k*s*zeta)*sin(j*zeta)*Mju1_wj3(ell-s+2*m-1) + (1-exp(1i*j*k*s*zeta)*cos(j*zeta))*Mju2_wj3(ell-s+2*m-1);
        dhj_dv3(ell+m,s+m) = (1-exp(1i*j*k*s*zeta))*Mju3_wj3(ell-s+2*m-1);
        
        dhj_dwj(ell+m,s+m) = 3*wj2_Mju1_Mjv1(ell-s+2*m-1) + 3*wj2_Mju2_Mjv2(ell-s+2*m-1) + 3*wj2_Mju3_Mjv3(ell-s+2*m-1) + 3*alpha(j)*wj2_ext(ell-s+2*m-1);
        
    end
    
    iDF(i_wj(j,:),i_u1) = dhj_du1;
    iDF(i_wj(j,:),i_u2) = dhj_du2;
    iDF(i_wj(j,:),i_u3) = dhj_du3;
    
    iDF(i_wj(j,:),i_v1) = dhj_dv1;
    iDF(i_wj(j,:),i_v2) = dhj_dv2;
    iDF(i_wj(j,:),i_v3) = dhj_dv3;
    
    iDF(i_wj(j,:),i_wj(j,:)) = diag(1i*(-m+1:m-1)) + dhj_dwj;
    
end

end
