function [convolution] = rig_conv_2sided(a)

% Inputs : (p vectors of Fourier coefficients - a1, ..., ap) 
%  ai = (ai_-M+1},...,ai_{-1},ai_0,ai_1,...,ai_{M-1}) \in C^{2M-1}, for i=1,...,p

% This code computes a rigorous enclosure of a discrete convolution 
% (a1*...*ap)_k = sum_{k1+...+kp=k, ki in Z} a1_k1 ... ap_kp for k=-p*(M-1),...,p*(M-1)

[n,p] = size(a); % p : power of the nonlinearity
M = (n+1)/2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% First enclosure: Rigorous estimates based on the FFT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We make sure that the inputs are powers of 2 %

exponent = ceil( log(2*p*M-1)/log(2) );
M1 = (2^exponent-2*p*M)/2; % Hence 2*p*M+2*M1 is a power of 2 %

ta = intval(zeros(2*p*M+2*M1,p));
tu = intval(zeros(2*p*M+2*M1,p));
tb = ones(2*p*M+2*M1,1);

for i=1:p
    ta(:,i) = [intval(zeros((p-1)*M+M1+1,1));a(:,i);intval(zeros((p-1)*M+M1,1))];
    tu(:,i) = verifyfft(ifftshift(ta(:,i)),-1);
    tb = tb.*tu(:,i);
end

F = fftshift(verifyfft(tb,1));

convolution1 = (2*p*M+2*M1)^(p-1)*F(M1+p+1:M1+p*(2*M-1)+1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Second enclosure: Rigorous estimates based on Banach Algebras
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rho = zeros(p,1);

for i=1:p
    rho(i) = exp_decay_a_least_square(mid(a(M:2*M-1,i)));
end

rho_min = intval(min(rho));
omega = [intval(1) 2*rho_min.^abs(1:M-1)]';

ia = intval(a);
norm_product = 1;

for i=1:p
    norm_product = norm_product*sum(abs(ia(M:2*M-1,i)).*omega);
end

k_p = (1:p*(M-1));
omega = [intval(1) 2*rho_min.^k_p]';

convolution2 = sup(norm_product./omega);

convolution2 = infsup(-convolution2,convolution2);

convolution2 = [flip(convolution2(2:end));convolution2];

convolution2 = convolution2 + 1i*convolution2;

convolution = intersect(convolution1,convolution2);


end