function [x_sym,tu_sym] = symmetrize_x(x,tu,par)

% Inputs: x = [lambda;alpha;u;v;w]; 
%               lambda = (lambda_1,lambda_2,lambda_3) in C^3 the unfolding
%                        parameters for the three conserved quantities
%               alpha = (alpha_1,...,lambda_{n-1}) in C^{n-1} the unfolding
%                        parameters for the n-1 extra initial conditions
%               u = (u_1,u_2,u_3) in (ell_nu^1)^3 the Fourier coefficients
%                   of the original phase space variables u(t) = (x(t),y(t),z(t))
%               v = (v_1,v_2,v_3) in (ell_nu^1)^3 the Fourier coefficients
%                   of the derivative v(t) = u'(t) = (x'(t),y'(t),z'(t))
%               w = (w_1,...,w_{n-1}) in (ell_nu^1)^{n-1} the Fourier coefficients
%                   of the nonlinearity functions w_j(t)
%         tu = (tu_1,tu_2,tu_3) the Fourier coefficients of a periodic
%              solution previsouily obtained 
%         par = [omega;n;k];
%

n = par(2); % Number of bodies in the choreography

% lambda in C^3: unfolding parameters for the three conserved quantities
lambda = x(1:3); 

%alpha in C^{n-1}: unfolding parameters for the n-1 extra init. conditions
alpha = x(4:n+2);

m = (length(x)+3)/(2*(n+5)); % The Fourier projection per component

i1 = (1:2*m-1)'; i2 = i1+2*m-1; i3 = i2+2*m-1;

tu_sym = [symmetrize_a(tu(i1));symmetrize_a(tu(i2));symmetrize_a(tu(i3))];

%u : Fourier coefficients of u(t) = (x(t),y(t),z(t))
u = reshape(x(n+3:6*m+n-1),2*m-1,3); %u_1 = u(:,1); u_2 = u(:,2); u_3 = u(:,3);
u1 = symmetrize_a(u(:,1)); u2 = symmetrize_a(u(:,2)); u3 = symmetrize_a(u(:,3));
u_sym = [u1;u2;u3];

%v : Fourier coefficients of v(t) = u'(t) = (x'(t),y'(t),z'(t))
v = reshape(x(6*m+n:12*m+n-4),2*m-1,3); 
v1 = symmetrize_a(v(:,1)); v2 = symmetrize_a(v(:,2)); v3 = symmetrize_a(v(:,3));
v_sym = [v1;v2;v3];

%w : Fourier coefficients of w(t) = (w_1(t),...,w_{n-1}(t))
w = reshape(x(12*m+n-3:2*m*(n+5)-3),2*m-1,n-1); %w_1 = w(:,1); ... w_{n-1} = w(:,n-1);

w_sym = zeros((2*m-1)*(n-1),1);

index = (1:2*m-1)';

for j = 1 : n-1
    
    w_sym(index) = symmetrize_a(w(:,j));
         
    index = index + 2*m-1;
   
end

x_sym = [real(lambda);real(alpha);u_sym;v_sym;w_sym];

end

function [a_sym] = symmetrize_a(a)

m = (length(a)+1)/2;

a_sym = [flip(conj(a(m+1:2*m-1)));real(a(m));a(m+1:2*m-1)];

end

